import React, { useEffect, useState, useRef } from 'react';
import {
  Col,
  Modal,
  Row,
  Image,
  Avatar,
  Space,
  Card,
  Descriptions,
  Tag,
  Spin,
  Input,
  Popconfirm,
  message,
} from 'antd';
import { getAgentDetail } from '@/services/agent';

import { editAgentInfo } from '@/services/agent';

import type { roleListType } from './index';

type DetailModalType = {
  visible: boolean;
  id: string;
  closeModal: () => void;
};

type followBarsType = {
  avatarUrl: string;
  label: string;
  [key: string]: any;
};

type userInfoType = {
  id: string;
  avatarUrl: string;
  nickName: string;
  authRoles: any;
  phoneNumber: string;
  followBarNum: number | null;
  followBars: followBarsType[];
  tags: string[] | null;
  [key: string]: any;
};

const tabList = [
  {
    key: 'tieba',
    tab: '关注的吧',
  },
  {
    key: 'tag',
    tab: '角色标签',
  },
];

const newTagName = '+ 新增标签';

const DetailModal: React.FC<DetailModalType> = ({ visible, id, closeModal }) => {
  const [userInfo, setUserInfo]: [userInfoType, (params: userInfoType) => void] = useState<
    Record<string, any>
  >({});

  const [spinning, setSpinning] = useState<boolean>(false);

  const [tabKey, setTabKey] = useState<'tieba' | 'tag'>('tieba');

  const [tagModalVisable, setTagModalVisable] = useState<boolean>(false);

  const [tagName, setTagName] = useState<string>('');

  // 获取详情
  const GetAgentDetail = async () => {
    setSpinning(true);
    const { code, data } = await getAgentDetail(id);
    setSpinning(false);
    if (code === 0) {
      setUserInfo(data);
      console.log(userInfo.authRoles, '获取详情');
    }
  };

  useEffect(() => {
    GetAgentDetail();
  }, []);

  // 渲染关注的贴吧
  const renderFollowBar = (): React.ReactNode => {
    return (
      <Row gutter={[16, 16]}>
        {userInfo.followBars && userInfo.followBars.length > 0 ? (
          userInfo.followBars.map((item) => (
            <Col key={item.id} span={4}>
              <Space direction="vertical" align="center">
                <Image
                  style={{ objectFit: 'cover' }}
                  width={50}
                  height={50}
                  src={item.avatarUrl}
                  alt="我炸裂了"
                />
                <span>{item.label}</span>
              </Space>
            </Col>
          ))
        ) : (
          <div>暂无关注的吧</div>
        )}
      </Row>
    );
  };

  // 打开新增标签弹窗
  const openTagModal = () => setTagModalVisable(true);

  // 关闭新增标签弹窗
  const closeTagModal = () => setTagModalVisable(false);

  // 新增标签
  const addCustomTag = () => {};
  // 删除标签
  const clearTags = (e: React.MouseEvent<HTMLElement, MouseEvent> | undefined, _tag: string) => {
    e && e.preventDefault();

    const tags = userInfo.tags == null ? [] : userInfo.tags;

    const residueTags = tags.filter((tag) => tag !== _tag && tag !== newTagName);

    EditAgentInfo(residueTags, 'delete');
  };

  // 添加标签
  const addTags = () => {
    const tags = userInfo.tags == null ? [] : userInfo.tags;

    const residueTags = tags.filter((_tag) => _tag !== newTagName);

    residueTags.push(tagName);

    EditAgentInfo(residueTags, 'add');
  };

  // 更改代理商信息
  const EditAgentInfo = async (residueTags: string[], type: 'delete' | 'add') => {
    const $par = {
      ...userInfo,
      tags: residueTags,
    };

    const { data, code } = await editAgentInfo($par);

    message.success(type === 'add' ? '增加成功' : '删除成功');

    if (code === 0 && data) {
      setUserInfo({
        ...userInfo,
        tags: residueTags,
      });

      setTagModalVisable(false);

      setTagName('');
    }
  };

  const changeTags = (e) => {
    console.log(e.target.value);

    setTagName(e.target.value);
  };

  // 渲染自定义标签
  const renderCustomTag = () => {
    const redisueTagCount =
      userInfo.tags && userInfo.tags.filter((tag) => tag !== newTagName).length;

    let tags: string[] = [];

    if (redisueTagCount === 2) {
      tags = userInfo.tags;
    } else if (redisueTagCount === 1) {
      tags = userInfo.tags.concat(newTagName);
    } else {
      tags = [newTagName];
    }

    return (
      <>
        {tags.map((tag, index) => (
          <a key={tag + index}>
            {tag === newTagName ? (
              <Tag closable={false} onClick={openTagModal}>
                {tag}
              </Tag>
            ) : (
              <Popconfirm title="确定要删除这个标签吗？" onConfirm={(e) => clearTags(e, tag)}>
                <Tag color="blue">{tag}&nbsp; x</Tag>
              </Popconfirm>
            )}
          </a>
        ))}
        <Modal title="新增标签" visible={tagModalVisable} onCancel={closeTagModal} onOk={addTags}>
          <Input
            width={100}
            value={tagName}
            onChange={changeTags}
            maxLength={4}
            placeholder="请填写标签内容(最多四个字)"
          />
        </Modal>
      </>
    );
  };

  // 渲染用户角色
  const renderAuthRoles = (): React.ReactNode => {
    return (
      <Space>
        {userInfo.authRoles.map((item: roleListType) => (
          <Tag key={item.id} color="magenta">
            {item.roleLabel}
          </Tag>
        ))}
      </Space>
    );
  };

  return (
    <Modal title="详细信息" visible={visible} width={1200} onOk={closeModal} onCancel={closeModal}>
      <Spin spinning={spinning}>
        <Row gutter={16}>
          <Col span={8}>
            <Space direction="vertical" align="center">
              <Avatar
                src={userInfo.avatarUrl}
                alt="我炸裂了"
                size={{ xs: 24, sm: 32, md: 40, lg: 64, xl: 80, xxl: 100 }}
              />
              <Descriptions layout="horizontal" column={1}>
                <Descriptions.Item span={1} label="姓名">
                  {userInfo.nickName}
                </Descriptions.Item>
                <Descriptions.Item span={1} label="手机号">
                  {userInfo.phoneNumber}
                </Descriptions.Item>
                <Descriptions.Item span={1} label="地址">
                  {userInfo.region || '暂无地址'}
                </Descriptions.Item>
                <Descriptions.Item span={1} label="角色">
                  {userInfo.authRoles && renderAuthRoles()}
                </Descriptions.Item>
              </Descriptions>
            </Space>
          </Col>
          <Col span={16}>
            <Card
              tabList={tabList}
              style={{ width: '100%', height: '100%' }}
              activeTabKey={tabKey}
              onTabChange={(key) => setTabKey(key as 'tieba' | 'tag')}
            >
              {tabKey === 'tieba' ? renderFollowBar() : renderCustomTag()}
            </Card>
          </Col>
        </Row>
      </Spin>
    </Modal>
  );
};
export default DetailModal;
