import React, { Component } from 'react';
import { Input, Space, Row, Col, Card, Button, Table, Modal, message, Popconfirm } from 'antd';

import { PageContainer } from '@ant-design/pro-layout';

import { getClassify, postOrPutClassify, deleteClassify } from '@/services/tieba';

import { filterTimestamp } from '@/filters/index';

import { ConnectState } from '@/models/connect';

import { connect, Dispatch } from 'umi';

import { Role } from '@/models/user';

const { Search } = Input;

type ModalEleType = {
  visible: boolean;
  addOrEditClassify: () => Promise<void>;
  closeModal: () => void;
  changeLabel: (value: string) => void;
  loading: boolean;
  label: string;
  type: number;
};

const ModalEle: React.FC<ModalEleType> = ({
  visible,
  addOrEditClassify,
  closeModal,
  changeLabel,
  type,
  loading,
  label,
}) => {
  const onChange = (e: React.ChangeEvent<HTMLInputElement>) => changeLabel(e.target.value);

  return (
    <Modal
      confirmLoading={loading}
      visible={visible}
      title={type === 0 ? '新增类目' : '编辑类目'}
      width={400}
      onOk={addOrEditClassify}
      onCancel={closeModal}
    >
      <Row align="middle">
        <Col span={4}>类目名:</Col>
        <Col span={16}>
          <Input maxLength={8} value={label} placeholder="最多8个字符" onChange={onChange}></Input>
        </Col>
      </Row>
    </Modal>
  );
};

interface IProps {
  dispatch: Dispatch;
  roleList: Role[];
}

class Classify extends Component<IProps> {
  state = {
    queryParams: {
      curPage: 1,
      label: '',
      total: 0,
      type: 0,
      id: '',
      roleId: Math.random().toString(36).substring(2),
    },
    visible: false, // 弹窗
    loading: false,
    classifyList: [],
  };

  componentDidMount() {
    this.props.dispatch({
      type: 'user/getRoleList',
      payload: {
        curPage: 1,
        passSize: 100,
      },
      callback: () => {
        console.log(this.props.roleList, 'this.props.roleList');
        this.setState({
          ...this.state,
          queryParams: {
            ...this.state.queryParams,
            roleId: this.props.roleList[0].id,
          },
        });
      },
    });

    this.getClassify();
  }

  // change label
  changeLabel = (value: string): void =>
    this.setState({ queryParams: { ...this.state.queryParams, label: value } });

  // 获取贴吧分类
  async getClassify(): Promise<void> {
    this.setState({ loading: true });
    const { code, data } = await getClassify(this.state.queryParams);
    this.setState({ loading: false });
    if (code === 0) {
      const { records, total } = data;
      this.setState({
        classifyList: records,
        queryParams: { ...this.state.queryParams, total },
      });
    }
  }

  // 新增或者修改类目名
  addOrEditClassify = async (): Promise<void> => {
    const { type } = this.state.queryParams;
    this.setState({ loading: true });
    const { code } = await postOrPutClassify(this.state.queryParams);
    this.setState({ loading: false });
    if (code === 0) {
      message.success(`${type === 0 ? '新增' : '编辑'}成功`);

      this.setState({
        queryParams: { ...this.state.queryParams, label: '', id: '' },
      });

      this.getClassify();
    }
    this.closeModal();
  };
  // 删除分类
  deleteClassify = async (record: Record<string, any>): Promise<void> => {
    const { code, data } = await deleteClassify(record.id);
    if (code === 0) {
      console.log(data);
      message.success('删除成功');
    }
    this.getClassify();
  };

  // 打开弹窗
  openModal = (type: number, record?: Record<string, any>): void => {
    this.setState({
      visible: true,
      queryParams: {
        ...this.state.queryParams,
        type,
        label: record?.label,
        id: type === 1 ? record?.id : '',
      },
    });
  };

  // 关闭弹窗
  closeModal = (): void => this.setState({ visible: false });

  // 分页
  changePagination = (page: number): void => {
    this.setState({ queryParams: { ...this.state.queryParams, curPage: page } }, () => {
      this.getClassify();
    });
  };

  // 搜索
  onSearch = (value: string): void => {
    this.setState(
      {
        queryParams: { ...this.state.queryParams, label: value, curPage: 1 },
      },
      () => {
        this.getClassify();
      },
    );
  };

  // 切换tabs
  onTabChange = (activeKey: string) => {
    console.log(activeKey, 'activeKey');
    this.setState(
      {
        ...this.state,
        queryParams: {
          ...this.state.queryParams,
          roleId: activeKey,
        },
      },
      () => {
        this.getClassify();
      },
    );
  };

  render() {
    const { classifyList, visible, queryParams, loading } = this.state;
    const { type, total, label, roleId } = queryParams;
    const { roleList } = this.props;
    const {
      onSearch,
      openModal,
      addOrEditClassify,
      closeModal,
      changeLabel,
      changePagination,
      deleteClassify,
      onTabChange,
    } = this;

    return (
      <PageContainer
        title="贴吧的分类管理"
        tabActiveKey={roleId}
        tabList={roleList.map((role) => {
          return {
            key: role.id,
            tab: role.label,
          };
        })}
        onTabChange={onTabChange}
      >
        <Card>
          <Row justify="space-between">
            <Col>
              <Search placeholder="输入分类名称" onSearch={onSearch} enterButton />
            </Col>
            <Col>
              <Button type="primary" onClick={() => openModal(0)}>
                新增分类
              </Button>
            </Col>
          </Row>

          <Table
            loading={loading}
            style={{ marginTop: 20 }}
            dataSource={classifyList}
            pagination={{ total, onChange: changePagination }}
          >
            <Table.Column title="类目名" key="label" dataIndex="label"></Table.Column>
            <Table.Column title="创建人" key="creator" dataIndex="creator"></Table.Column>
            <Table.Column
              title="创建时间"
              key="createTime"
              dataIndex="createTime"
              render={(text) => <span>{filterTimestamp(text)}</span>}
            ></Table.Column>
            <Table.Column
              title="操作"
              key="action"
              dataIndex="action"
              render={(text, record) => (
                <Space>
                  <a onClick={() => openModal(1, record as Record<string, any>)}>编辑</a>
                  <Popconfirm
                    title="确定要删除吗？"
                    onConfirm={() => deleteClassify(record as Record<string, any>)}
                    okText="Yes"
                    cancelText="No"
                  >
                    <a>删除</a>
                  </Popconfirm>
                </Space>
              )}
            ></Table.Column>
          </Table>
        </Card>
        {/* 弹窗 */}
        <ModalEle
          type={type}
          label={label}
          loading={loading}
          changeLabel={changeLabel}
          visible={visible}
          addOrEditClassify={addOrEditClassify}
          closeModal={closeModal}
        />
      </PageContainer>
    );
  }
}

// Classify

export default connect(({ user }: ConnectState) => ({
  roleList: user.roleList,
}))(Classify);
